from faicons import icon_svg
from shiny import ui

from .utils import get_sheets

# use a CDN for assets instead of local
loading_tag = ui.tags.img(src="https://gilded-mandazi-30bb2e.netlify.app/loading.svg", height="32px", width="32px")
woman_on_smartphone = ui.tags.img(src="https://gilded-mandazi-30bb2e.netlify.app/woman_on_smartphone.svg")
woman_reminding = ui.tags.img(src="https://gilded-mandazi-30bb2e.netlify.app/woman_reminding.svg")


def record_observation(survey_config):
    return ui.nav_panel(
        "Record observation",
        ui.value_box(
            "Use this page to record your observations",
            "",
            "Once observations for a survey point are recorded, proceed to next page to upload observations \n",
            showcase=woman_on_smartphone,
            theme=("purple"),
            showcase_layout="left center",
        ),
        ui.layout_columns(
            ui.card(
                ui.accordion(
                    ui.accordion_panel(
                        "Select Data source",
                        ui.tooltip(
                            ui.input_switch(
                                "low_data_mode",
                                "Bandwidth Saver",
                                False,
                            ),
                            "Enable this mode to reduce your data usage.💰 ",
                            "It won't load images and trivia about specimen when turned on",
                            placement="top",
                            id="low_data_mode_tooltip",
                        ),
                        ui.input_radio_buttons(
                            id="data_source",
                            label="Select data source",
                            choices=survey_config["survey_data_sources"],
                        ),
                        ui.input_action_button(
                            "go_to_surveyors",
                            label="Proceed to next step",
                            class_="btn btn-info",
                            icon=icon_svg("circle-check"),
                        ),
                        value="data_source",
                        icon=icon_svg("database"),
                    ),
                    ui.accordion_panel(
                        "Select Surveyor details",
                        ui.input_date("survey_date", label="Survey Date"),
                        ui.input_selectize(
                            "location",
                            label="Select the location",
                            choices=survey_config["location"],
                        ),
                        ui.input_selectize(
                            "surveyors",
                            label="Choose Surveyor(s)*:",
                            choices=survey_config["surveyors"],
                            multiple=True,
                        ),
                        ui.input_action_button(
                            "go_to_survey_points",
                            label="Proceed to next step",
                            class_="btn btn-info",
                            icon=icon_svg("circle-check"),
                        ),
                        value="surveyors",
                        icon=icon_svg("people-group"),
                    ),
                    ui.accordion_panel(
                        "Select Survey site",
                        ui.input_text("gps_loc", label="GPS Location"),
                        ui.output_text("note_accuracy"),
                        ui.tooltip(
                            ui.input_task_button(
                                "gps",
                                "Get my location",
                                class_="btn btn-info",
                                icon=icon_svg("location-crosshairs"),
                                label_busy="Fetching...",
                                icon_busy=loading_tag,
                                auto_reset=False,
                            ),
                            "🎯 Press button multiple times to enhance accuracy within 15 meters. "
                            "Requires location access on browser",
                            id="gps_location_tooltip",
                        ),
                        ui.br(),
                        ui.br(),
                        ui.input_selectize(
                            "plot",
                            label="Select the plot",
                            choices=survey_config["plots"],
                        ),
                        ui.input_selectize(
                            "survey_point",
                            label="Select the survey point",
                            choices=survey_config["survey_points"],
                            options={"readonly": "readonly"},
                        ),
                        ui.input_selectize(
                            "survey_side",
                            label="Select the side",
                            choices=survey_config["survey_side"],
                        ),
                        ui.input_action_button(
                            "go_to_specimen",
                            label="Proceed to next step",
                            class_="btn btn-info",
                            icon=icon_svg("circle-check"),
                        ),
                        value="survey_points",
                        icon=icon_svg("location-dot"),
                    ),
                    ui.accordion_panel(
                        "Select Survey data",
                        ui.input_switch(
                            "alpha_code",
                            "Show 4-letter Alpha Code instead of Common Name",
                            False,
                        ),
                        ui.input_switch(
                            "genus_species",
                            "Show Genus + Species instead of Common Name",
                            False,
                        ),
                        ui.input_selectize(
                            "specimen", label="Select the specimen observed", choices={}
                        ),
                        ui.input_selectize(
                            "count",
                            label="Count observed",
                            choices=survey_config["count"],
                        ),
                        ui.input_radio_buttons(
                            "plant_survey",
                            "Is it a vegetation survey?",
                            ["Yes", "No"],
                            selected="No",
                            inline=True,
                        ),
                        ui.panel_conditional(
                            "input.plant_survey === 'Yes'",
                            ui.input_selectize(
                                "canopy_cover",
                                "Select the canopy cover",
                                choices=survey_config["canopy_cover"],
                            ),
                            ui.input_selectize(
                                "abundance",
                                label="Select the abundance",
                                choices=survey_config["abundance_category"],
                            ),
                            ui.input_selectize(
                                "life_stage",
                                "Select the life stage",
                                choices=survey_config["life_stage"],
                            ),
                            ui.input_numeric(
                                "height",
                                "Enter the height of the plant (in cm)",
                                0,
                                min=0,
                                max=1000,
                                step=5,
                            ),
                        ),
                        ui.input_text("notes", label="Notes"),
                        value="specimen_details",
                        icon=icon_svg("clipboard"),
                    ),
                    id="accordion_data_sources",
                    multiple=False,
                ),
                ui.tooltip(
                    ui.input_file(
                        "image_file",
                        "Upload Image file(s) for specimen (optional)",
                        accept=[".jpg", ".png", ".jpeg"],
                        multiple=True,
                    ),
                    "Uploads image to imgur.com",
                    id="upload_image_tooltip",
                ),
                ui.tooltip(
                    ui.input_task_button(
                        "submit",
                        "Record observation",
                        class_="btn btn-success",
                        icon=icon_svg("check"),
                        label_busy="Recording...",
                        icon_busy=loading_tag,
                    ),
                    "Saves the observation locally without uploading",
                    id="submit_observation_tooltip",
                ),
                ui.input_action_button(
                    "next_screen",
                    "Proceed to uploading observations",
                    class_="btn btn-outline-primary",
                    icon=icon_svg("arrow-right"),
                ),
                fillable=True,
                inline=True,
            ),
            ui.card(ui.output_ui("notes", fillable=True, inline=True)),
            col_widths=[3, 9],
            class_="shiny-input-full-width",
        ),
        icon=icon_svg("magnifying-glass"),
        value="record",
    )


def verify_observation(survey_config, keyfile_path):
    return (
        ui.nav_panel(
            "Upload observations",
            ui.value_box(
                "Remember to upload your data!",
                "",
                ui.markdown(
                    "To ensure your field data is not lost, please take a minute to upload your data to Google Sheets by clicking on the **ϟ Upload all observations** button.",
                ),
                showcase=woman_reminding,
                theme="bg-gradient-orange-yellow",
                id="reminder",
            ),
            ui.card(
                ui.tags.script(
                    """
                    let isWide = null;
                    function updateIsWide() {
                        const prevIsWide = isWide;
                        isWide = window.innerWidth > 600;
                        if (prevIsWide !== isWide) {
                            Shiny.setInputValue("isWide", isWide);
                        }
                    }
                    const setIsWide = function() {
                        if (Shiny.setInputValue) {
                            updateIsWide();
                        } else {
                            setTimeout(setIsWide, 2);
                        }
                    }
                    $( window ).on( "resize", setIsWide );
                    setIsWide();
                    """
                ),
                ui.panel_conditional(
                    "input.isWide",
                    ui.TagList(
                        ui.output_data_frame("observations_data_frame"),
                        ui.br(),
                        ui.br(),
                        ui.tooltip(
                            ui.input_action_button(
                                "reset",
                                "Clear selected observation",
                                class_="btn btn-outline-danger",
                                icon=icon_svg("snowplow"),
                                width="300px",
                            ),
                            "In case user entered an observation in error",
                            id="clear_observation_tooltip",
                        ),
                    ),
                ),
                ui.panel_conditional(
                    "!input.isWide", ui.output_ui("observations_list")
                ),
            ),
            ui.page_fluid(
                ui.card(
                    ui.card(
                        ui.input_radio_buttons(
                            id="google_sheet_selector",
                            label="Select Google Sheet to upload data",
                            choices=get_sheets(survey_config, keyfile_path),
                            selected=None,
                            inline=True,
                        ),
                        id="google_sheets_selector",
                    ),
                    ui.tooltip(
                        ui.input_task_button(
                            "upload",
                            "Upload all observations",
                            class_="btn btn-success",
                            icon=icon_svg("bolt-lightning"),
                            width="300px",
                            label_busy="uploading...",
                            icon_busy=loading_tag,
                        ),
                        "Saves all the observations to the database",
                        id="upload_all_observations",
                    ),
                    ui.markdown(
                        f"""
            Find all uploaded observations in this [link]({survey_config["database_link"]})
            """
                    ),
                    ui.input_action_button(
                        "previous_screen",
                        "Go back to recording observation(s)",
                        class_="btn btn-outline-primary",
                        icon=icon_svg("arrow-left"),
                        width="300px",
                    ),
                    ui.markdown(
                        """
                    If your observations got lost because <br>
                    of a bad connection or page refresh before uploading, don't worry!<br>
                    You can bring them back by clicking<br>
                    the **"Restore Observation(s)"** button below.
                    """
                    ),
                    ui.tooltip(
                        ui.input_task_button(
                            "restore",
                            "Restore observation(s)",
                            class_="btn btn-warning",
                            icon=icon_svg("trash-arrow-up"),
                            width="300px",
                            label_busy="Restoring...",
                            icon_busy=loading_tag,
                        ),
                        "In case you want to restore an observation which was lost before it could be uploaded",
                        id="restore_observation_tooltip",
                    ),
                    ui.tags.script(
                        """
                        $(document).ready(function () {
                        Shiny.addCustomMessageHandler("restore", function (message) {

                            if (message === "restore") {
                                Shiny.setInputValue("restore_df", localStorage.getItem("df"));
                            } else if (message === "complete") {
                                Shiny.setInputValue("restore_df", null);
                                }

                        });
                        

                        Shiny.addCustomMessageHandler("df", function (message) {
                            localStorage.setItem("df", message);
                            Shiny.setInputValue("local_storage_df", localStorage.getItem("df"));
                        });


                        Shiny.addCustomMessageHandler("clear", function (message) {
                            if (message === "clear") {
                            console.log("clear the store");
                                localStorage.removeItem("df");
                                return;
                            }
                        });
                        });
                    """
                    ),
                ),
            ),
            icon=icon_svg("cloud-arrow-up"),
            value="verify",
        ),
    )



import json

import gspread
import requests
import us
import wikipediaapi
from bs4 import BeautifulSoup
from cryptography.fernet import Fernet
from geopy.geocoders import Nominatim

# instantiate a Nominatim geocoder
geolocator = Nominatim(user_agent="temp_shiny_app")


def get_species_image(genus, species, image_number, family=None, order=None):
    """
    Retrieves the image URL for a given species.

    Args:
        genus (str): The genus of the species.
        species (str): The species name.
        image_number (int): The number of the image to retrieve. The first image is 0, the second is 1, etc.

    Returns:
        str: The URL of the species image, or a default image URL if no image is found.
    """
    base_url = "https://api.inaturalist.org/v1/taxa/autocomplete"
    query = (
        order
        if order is not None
        else family if family is not None else f"{genus} {species}"
    )
    params = {
        "q": query,
        "limit": 1,  # Limit to the first result
    }

    response = requests.get(base_url, params=params)

    if response.status_code == 200:
        data = response.json()
        if data["results"] and data["results"][0]["id"]:
            taxon_id = data["results"][0]["id"]
            photo_url = f"https://api.inaturalist.org/v1/taxa/{taxon_id}?locale=en"
            photo_response = requests.get(photo_url)
            if photo_response.status_code == 200:
                photo_data = photo_response.json()
                if (
                    photo_data["results"]
                    and photo_data["results"][0]["taxon_photos"]
                    and len(photo_data["results"][0]["taxon_photos"]) > 1
                ):
                    if (
                        "large_url"
                        in photo_data["results"][0]["taxon_photos"][1]["photo"]
                    ):
                        image_url = photo_data["results"][0]["taxon_photos"][
                            image_number
                        ]["photo"]["large_url"]
                        return image_url

    return "https://i.ibb.co/m6YDp69/sorry.jpg"


# use this if you want to use the openweathermap API
def get_current_temperature(survey_config, latitude, longitude):
    base_url = "http://api.openweathermap.org/data/2.5/weather"
    params = {
        "lat": latitude,
        "lon": longitude,
        "appid": survey_config["openweathermap_api_id"],
        "units": "imperial",  # You can change this to 'metric' for Celcius
    }

    try:
        response = requests.get(base_url, params=params)
        data = response.json()

        if response.status_code == 200:
            temperature = data["main"]["temp"]
            return f"{temperature} °F"
        else:
            print(f"Error: {data['message']}")
            return None
    except Exception as e:
        print(f"Error fetching data: {e}")
        return None


def get_summary_for_specimen(genus, species):
    wiki_wiki = wikipediaapi.Wikipedia("Survey App Ecology (merlin@example.com)", "en")
    page_py = wiki_wiki.page(f"{genus} {species}")

    if page_py.exists():
        return page_py.summary
    else:
        return "No notes found"


def fetch_specimen_classification(genus, species, common_name):
    """
    Fetches the classification information for a given specimen based on its genus, species, or common name.

    Args:
        genus (str): The genus of the specimen.
        species (str): The species of the specimen.
        common_name (str): The common name of the specimen.

    Returns:
        dict: A dictionary containing the classification information of the specimen, including the family, order, and class.

    """
    results = {
        "genus": genus if genus else "unknown",
        "species": species if species else "unknown",
        "family": "unknown",
        "order": "unknown",
        "class": "unknown",
    }

    if genus and genus not in ["None", "Unknown", "unknown", "other"]:
        if species and species not in ["None", "Unknown", "unknown", "other"]:
            species_scientific_name = f"{genus} {species}"
            url = f"https://api.inaturalist.org/v1/taxa?q={species_scientific_name}"
        else:
            url = f"https://api.inaturalist.org/v1/taxa?q={genus}"

    elif common_name and common_name.lower() in ["none", "unknown"]:
        return results
    elif "unknown" in common_name.lower():
        common_name = common_name.lower().replace("unknown", "").strip()
        url = f"https://api.inaturalist.org/v1/taxa?q={common_name}"
    else:
        return results

    response = requests.get(url)
    if response.json()["results"][0]:
        data = response.json()["results"][0]
        url1 = f"https://api.inaturalist.org/v1/observations?verifiable=true&taxon_id={data['id']}"

        response = requests.get(url1)
        data = response.json()
        ranks = data["results"][0]["identifications"][0]["taxon"]["ancestors"]

        for rank in ranks:
            if rank["rank"] == "family":
                results["family"] = rank["name"]
            if rank["rank"] == "order":
                results["order"] = rank["name"]
            if rank["rank"] == "class":
                results["class"] = rank["name"]
    return results


# this approach helps get weather without an API key
def get_weather_underground_temperature(latitude, longitude):
    location = geolocator.reverse(latitude + "," + longitude)
    address = location.raw["address"]
    country_code = address.get("country_code", "")
    city = (
        address.get("city", address.get("village", address.get("hamlet", "")))
        .lower()
        .replace(" ", "-")
    )
    if country_code == "us":
        state = us.states.lookup(address.get("state", "")).abbr.lower()
        url = f"https://www.wunderground.com/weather/{country_code}/{state}/{city}"
    else:
        state = address.get("state", "").lower()
        url = f"https://www.wunderground.com/weather/{country_code}/{city}"

    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3"
    }
    page = requests.get(url, headers=headers)
    soup = BeautifulSoup(page.content, "html.parser")

    weather = soup.find(class_="current-temp")
    weather_text = weather.text
    return weather_text


def decrypt_file(encrypted_file_path, key):
    with open(encrypted_file_path, "rb") as encrypted_file:
        encrypted_data = encrypted_file.read()

    cipher_suite = Fernet(key)
    decrypted_data = cipher_suite.decrypt(encrypted_data)

    return decrypted_data


def get_sheets(survey_config, keyfile_path):
    google_worksheets = []
    sh = get_workbook(survey_config, keyfile_path)
    for sheet in sh.worksheets():
        google_worksheets.append(sheet.title)
    return google_worksheets


def get_workbook(survey_config, keyfile_path):
    with open(keyfile_path, "r") as file:
        credentials = json.load(file)
    gc = gspread.service_account_from_dict(credentials)
    sh = gc.open(survey_config["google_workbook_name"])
    return sh



from fieldsurveys.app_files.image_upload import upload_image_to_drive

# TODO: Fix tests
def test_upload_image_to_drive_failure(
    monkeypatch,
):
    monkeypatch.setattr(
        "fieldsurveys.app_files.image_upload.service_account.Credentials.from_service_account_file",
        lambda *args, **kwargs: "mock_creds",
    )
    monkeypatch.setattr(
        "fieldsurveys.app_files.image_upload.build", lambda *args, **kwargs: mock_build
    )
    monkeypatch.setattr(
        "fieldsurveys.app_files.image_upload.MediaFileUpload",
        lambda *args, **kwargs: mock_media_file_upload,
    )
    monkeypatch.setattr(
        "fieldsurveys.app_files.image_upload.service.permissions",
        lambda *args, **kwargs: mock_service_permissions,
    )

    survey_config = {"imgur_app_id": "test"}
    image_path = "tests/assets/sample.jpeg"
    keyfile_json = "path/to/keyfile.json"

    result = upload_image_to_drive(survey_config, image_path, keyfile_json)

    assert result is None
    mock_service_account.Credentials.from_service_account_file.assert_called_once_with(
        keyfile_json, scopes=["https://www.googleapis.com/auth/drive"]
    )
    mock_build.assert_called_once_with("drive", "v3", credentials="mock_creds")
    mock_media_file_upload.assert_called_once_with(
        image_path, mimetype="image/jpeg", resumable=True
    )
    mock_build.return_value.files.return_value.create.assert_called_once_with(
        body={"parents": ["your_folder_id"]},
        media_body=mock_media_file_upload.return_value,
        fields="id",
    )
    mock_service_permissions.return_value.create.assert_not_called()



import os
import time

import shutil
import questionary
from colorama import Style, Fore

from tkinter import filedialog
from tkinter import messagebox
from tkinter.messagebox import showwarning
import sys
from .make_app import make_app

__all__ = ["make_app"]


# supress TK_SILENCE_DEPRECATION=1 warning
os.environ["TK_SILENCE_DEPRECATION"] = "1"


def _copy_file(source, destination):
    shutil.copy(source, destination)


def progress_bar(duration):
    bar_width = 50

    interval = 0.1

    num_steps = int(duration / interval)

    for i in range(num_steps + 1):
        progress = i / num_steps
        bar = (
            "["
            + "#" * int(progress * bar_width)
            + "-" * (bar_width - int(progress * bar_width))
            + "]"
        )
        sys.stdout.write("\r" + "Progress: {:3.0f}% {}".format(progress * 100, bar))
        sys.stdout.flush()
        time.sleep(interval)

    print("\nCompleted!")


def show_information(title: str, message: str) -> None:
    messagebox.showinfo(title, message)


def copy_app_files(
    app_dir=None, config_file=None, data_files=None, keyfile=None, dest=None
):
    if app_dir is None:
        app_dir = questionary.text(
            "Please specify the desired name for the application",
            validate=lambda text: len(text) > 0,
        ).ask()

    if config_file is None:
        show_information(
            "yaml file",
            "Kindly select your survey.yaml file using the file selection tool",
        )
        filetypes = [("YAML files", "*.yaml")]
        config_file = select_file("survey.yaml", filetypes)

    if data_files is None:
        show_information(
            "csv file(s)",
            "Next, please choose all the necessary input CSV files by using the file selection tool",
        )
        filetypes = [("CSV files", "*.csv")]
        data_files = select_file("data csv", filetypes=filetypes, multiple=True)

    if keyfile is None:
        show_information(
            "Keyfile json",
            "Next, please select your keyfile.json file using the file selection tool",
        )
        filetypes = [("JSON files", "*.json")]
        keyfile = select_file("keyfile.json", filetypes)

    if dest is None:
        show_information(
            "Destination dir",
            "Now, please select the destination directory where you would like the application to be located",
        )
        dest = select_directory()

    default_survey_path = os.path.join(os.path.dirname(__file__), "survey.yaml")
    default_csv_dir = os.path.join(os.path.dirname(__file__), "data")
    default_keyfile_path = os.path.join(os.path.dirname(__file__), "keyfile.json")

    print(Style.BRIGHT + f"Creating app at {dest}")

    if app_dir and not os.path.exists(os.path.join(dest, app_dir)):
        os.makedirs(os.path.join(dest, app_dir))

    app_dir = os.path.join(dest, app_dir)

    if config_file is None:
        config_file = default_survey_path
        _copy_file(config_file, app_dir)
    else:
        for config in config_file:
            _copy_file(config, app_dir)

    data_directory_path = os.path.join(app_dir, "data")
    if not os.path.exists(data_directory_path):
        os.makedirs(data_directory_path)

    if data_files is None:
        files = os.listdir(default_csv_dir)
        for file in files:
            file_path = os.path.join(default_csv_dir, file)
            if os.path.isfile(file_path):
                _copy_file(file_path, data_directory_path)
    else:
        for data_file in data_files:
            _copy_file(data_file, data_directory_path)

        if keyfile is None:
            keyfile = default_keyfile_path
            _copy_file(keyfile, app_dir)
        else:
            for file in keyfile:
                _copy_file(file, app_dir)
                # rename the file to keyfile.json no matter what user specified
                os.rename(
                    os.path.join(app_dir, os.path.basename(keyfile)),
                    os.path.join(app_dir, "keyfile.json")
                )

    file_contents = """\
import fieldsurveys

app = fieldsurveys.make_app(survey_path="survey.yaml", csv_dir="data", keyfile_path="keyfile.json")
    """

    with open(os.path.join(app_dir, "app.py"), "w") as file:
        file.write(file_contents)

    requirements = """\
fieldsurveys
        """

    with open(os.path.join(app_dir, "requirements.txt"), "w") as file:
        file.write(requirements)

    progress_bar(1)
    print(Style.BRIGHT + f"App created at {app_dir}")
    print(
        Style.BRIGHT
        + f"Navigate to {app_dir} and run the command below to run the app locally"
    )
    print(Fore.YELLOW + "python -m shiny run app.py")


def select_directory() -> str:
    directory_path = filedialog.askdirectory(title="Select Directory")

    if directory_path:
        return directory_path
    else:
        showwarning(
            "No directory selected",
            "Directory needs to be selected in order to proceed",
        )
        sys.exit(1)


def select_file(file_type: str, filetypes: list, multiple: bool = False) -> str:
    file_path = filedialog.askopenfilenames(
        title=f"Select {file_type} File", filetypes=filetypes, multiple=multiple
    )

    if file_path:
        return file_path

    else:
        showwarning(
            "No file(s) selected",
            f"{file_type} file needs to be selected in order to customize app.\n Using default version instead",
        )
        return None



from fieldsurveys.app_files.utils import get_species_image


# TODO: Fix tests
def test_get_species_image_api_failure(monkeypatch):
    class MockResponse:
        def __init__(self, status_code):
            self.status_code = status_code

    def mock_get(*args, **kwargs):
        return MockResponse(500)

    monkeypatch.setattr("requests.get", mock_get)

    result = get_species_image("Genus", "Species", 0)

    # Assert that the result is the default image URL
    assert result == "https://i.ibb.co/m6YDp69/sorry.jpg"



from playwright.sync_api import Page, expect
import time
import pytest
import subprocess

PORT = "63092"


def dismiss_modal(page: Page) -> None:
    page.locator("#shiny-modal").press("Escape")
    time.sleep(0.5)  # TODO: This is added to prevent tests being too fast to click
    page.locator("#shiny-modal").click()


# TODO - Create a temp app
@pytest.fixture(scope="session")
def start_shiny_app():
    # Start the app locally
    shiny_app_process = subprocess.Popen(
        ["python", "-m", "shiny", "run", "--port", PORT],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    # Wait for the app to load successfully
    timeout = 30  # Timeout in seconds
    start_time = time.time()
    while True:
        output = shiny_app_process.stderr.readline().decode().strip()
        if "Application startup complete." in output:
            break
        if time.time() - start_time > timeout:
            raise TimeoutError("Timeout: Shiny app did not load within the specified time.")
        time.sleep(1)

    yield

    shiny_app_process.terminate()


@pytest.mark.only_browser("chromium")
def test_shiny_app(page: Page, start_shiny_app) -> None:
    page.goto(f"http://localhost:{PORT}/")

    # verify value box on landing page
    expect(page.get_by_text("Use this page to record your")).to_be_visible()
    # check if the first accordion is the data source accordion panel
    expect(page.locator("#accordion_data_sources")).to_contain_text(
        "Select Data source"
    )

    # disable bandwidth saver for test
    page.get_by_label("Bandwidth Saver").check()
    page.get_by_text("butterfly").click()
    page.get_by_role("button", name="Proceed to next step").click()

    # Assert that accordion panel title is updated
    expect(page.locator("#accordion_data_sources")).to_contain_text(
        "Data source: butterfly"
    )
    page.locator("#surveyors").locator("..").locator("> .multi").click()
    # Cole is the first one on the list
    page.get_by_text("Cole").click()
    page.get_by_label("Choose Surveyor(s)*:").press("Escape")
    page.get_by_role("button", name="Proceed to next step").click()
    expect(page.locator("#accordion_data_sources")).to_contain_text("Surveyors: Cole")

    # Location accordion panel
    page.get_by_text("GPS Location").click()
    expect(page.locator("#plot-label")).to_contain_text("Select the plot")
    page.get_by_role("button", name="Proceed to next step").click()
    expect(page.locator("#accordion_data_sources")).to_contain_text(
        "Survey Point: P1 - 0a"
    )

    # Specimen accordion panel

    # check if alpha code is toggled, genus+species selection is reset and vice versa
    page.get_by_label("Show 4-letter Alpha Code").check()
    expect(page.get_by_label("Show 4-letter Alpha Code")).to_be_checked()
    expect(page.get_by_label("Show Genus + Species instead")).not_to_be_checked()
    page.locator("#specimen").locator("..").locator("> .single").click()
    page.get_by_text("ADCA").click()
    page.get_by_label("Show Genus + Species instead").check()
    expect(page.get_by_label("Show 4-letter Alpha Code")).not_to_be_checked()
    expect(page.get_by_label("Show Genus + Species instead")).to_be_checked()
    page.locator("#specimen").locator("..").locator("> .single").click()
    page.get_by_text("Adelpha californica").click()
    page.get_by_label("Show Genus + Species instead").uncheck()
    expect(page.get_by_label("Show 4-letter Alpha Code")).not_to_be_checked()
    expect(page.get_by_label("Show 4-letter Alpha Code")).not_to_be_checked()

    # Look up by common name - most common use case
    page.locator("#specimen").locator("..").locator("> .single").click()
    page.get_by_text("Lange's Metalmark").click()
    page.get_by_role("button", name="Record observation").click()
    expect(page.get_by_role("heading")).to_contain_text("Verify your observation")
    expect(page.locator("#submit_verify")).to_contain_text("Looks good")
    expect(page.locator("#edit")).to_contain_text("Edit observation")
    page.get_by_role("button", name="Looks good").click()
    expect(page.get_by_text("Your observation has been")).to_be_visible()
    page.get_by_role("button", name="Proceed to uploading").click()

    # upload page
    expect(page.locator("tbody")).to_contain_text("Lange's Metalmark")
    page.get_by_role("cell", name="Lange's Metalmark").click()
    page.get_by_role("button", name="Clear selected observation").click()
    page.get_by_role("button", name="Delete").click()
    expect(page.locator("#shiny-modal")).to_contain_text(
        "Your observation has been cleared"
    )
    dismiss_modal(page)
    page.get_by_label("Select Google Sheet to upload").click()

    # check restore functionality
    page.get_by_role("button", name="Restore observation(s)").click()
    expect(page.locator("#shiny-modal")).to_contain_text(
        "Your observations have been restored"
    )
    dismiss_modal(page)
    expect(page.locator("tbody")).to_contain_text("Lange's Metalmark")
    page.get_by_role("button", name="Go back to recording").click()

    # record observations page -> assert restore remembered user choices
    expect(page.get_by_role("button", name="Surveyors: Cole")).to_be_visible()
    expect(page.get_by_role("button", name="Survey Point: P1 - 0a")).to_be_visible()
    expect(page.get_by_role("button", name="Data source: butterfly")).to_be_visible()
    page.get_by_role("button", name="Data source: butterfly").click()
    page.get_by_label("Bandwidth Saver").uncheck()
    page.get_by_role("button", name="Select Survey data").click()
    page.locator("#specimen").locator("..").locator("> .single").click()
    page.get_by_text("California Sister").click()
    expect(page.get_by_role("heading", name="Source: iNaturalist")).to_be_visible()
    expect(
        page.get_by_role("heading", name="California Sister pics").get_by_role(
            "insertion"
        )
    ).to_be_visible()
    expect(
        page.get_by_role("heading", name="California Sister notes").get_by_role(
            "insertion"
        )
    ).to_be_visible()
    expect(page.get_by_role("heading", name="Source: Wikipedia")).to_be_visible()

    # check vegetation survey conditional UI
    page.get_by_label("Yes").check()
    expect(page.get_by_text("Select the canopy cover")).to_be_visible()
    expect(page.get_by_text("Select the abundance")).to_be_visible()
    expect(page.get_by_text("Select the life stage")).to_be_visible()
    expect(page.get_by_text("Enter the height of the plant")).to_be_visible()
    page.get_by_label("No", exact=True).check()
    expect(page.get_by_text("Select the canopy cover")).not_to_be_visible()
    expect(page.get_by_text("Select the abundance")).not_to_be_visible()
    expect(page.get_by_text("Select the life stage")).not_to_be_visible()
    expect(page.get_by_text("Enter the height of the plant")).not_to_be_visible()



import ast
import datetime
import json
from pathlib import Path
from typing import List

import gspread_dataframe as gd
import pandas as pd
import pytz
import shinyswatch
import sys
import yaml
from faicons import icon_svg
from shiny import App, reactive, render, req, ui
from shiny.types import NavSetArg
from timezonefinder import TimezoneFinder

from fieldsurveys.app_files.image_upload import (
    upload_image_to_drive,
)
from fieldsurveys.app_files.navbar_utils import (
    record_observation,
    verify_observation,
    loading_tag,
)
from fieldsurveys.app_files.utils import (
    fetch_specimen_classification,
    get_species_image,
    get_summary_for_specimen,
    get_weather_underground_temperature,
    get_workbook,
)
import os

default_survey_path = os.path.join(os.path.dirname(__file__), "survey.yaml")
default_csv_dir = os.path.join(os.path.dirname(__file__), "data")
default_keyfile_path = os.path.join(os.path.dirname(__file__), "keyfile.json")


def make_app(
    survey_path: str = default_survey_path,
    *,
    csv_dir: str = default_csv_dir,
    keyfile_path: str = default_keyfile_path,
) -> App:
    with open(survey_path, "r") as file:
        survey_config = yaml.safe_load(file)

    required_fields = {
        "surveyors": "Surveyors",
        "location": "Location",
        "plots": "Plots",
        "google_workbook_name": "Google workbook name",
        "company_logo_url": "Company logo URL",
        "survey_data_sources": "Survey data sources",
        "database_link": "Database link",
        "google_folder_id": "Google folder ID",
    }

    for field, field_name in required_fields.items():
        if field not in survey_config or not survey_config[field]:
            print(f"{field_name} is not configured in the config file")
            sys.exit(1)

    # check if survey_data_sources values are the same as the file names in the data dir
    for data_source in survey_config.get("survey_data_sources", []):
        if not os.path.exists(f"{csv_dir}/{data_source}.csv"):
            print(f"Data source {data_source}.csv is missing in the data directory")
            print(
                "Either delete the data source in the csv or add the data source to the data directory"
            )
            sys.exit(1)

    def nav_controls(prefix: str) -> List[NavSetArg]:
        return [
            record_observation(survey_config),
            verify_observation(survey_config, keyfile_path),
        ]

    default_url = "https://i.imgur.com/e2IqrgC.png"
    img_url = (
        survey_config["company_logo_url"]
        if survey_config["company_logo_url"]
        else default_url
    )
    app_ui = ui.page_navbar(
        shinyswatch.theme.minty(),
        title=ui.tags.img(src=img_url, width="125px"),
        *nav_controls("page_navbar"),
        id="page_navbar",
        header=ui.tags.link(href="https://gilded-mandazi-30bb2e.netlify.app/styles.css", rel="stylesheet"),
        window_title="Survey App",
    )

    def server(input, output, session):
        new_df = pd.DataFrame(
            [],
            columns=[
                "Data Source",
                "Bandwidth Saver",
                "Date observed",
                "Location",
                "Plot",
                "Surveyors",
                "Survey Point",
                "Side",
                "Class",
                "Order",
                "Family",
                "Common Name",
                "Genus",
                "Species",
                "Count",
                "Notes",
                "Canopy cover",
                "Life stage",
                "Height (cm)",
                "Abundance",
            ],
        )
        val = reactive.Value(new_df)
        notes_val = reactive.Value(None)
        accuracy_val = reactive.Value(None)
        url_val = reactive.Value(None)
        index_to_delete_val = reactive.Value(None)

        @reactive.Effect
        async def _upload():
            df = data_source()
            notes_val.set(None)
            req(input.specimen())
            x = str(input.specimen())
            cols_to_check = ["Alpha Code", "Common Name", "Genus", "Species"]
            all_exist = all(elem in df.columns for elem in cols_to_check)

            if not df.empty and all_exist:
                if input.alpha_code():
                    req(any((df["Alpha Code"] == x)))
                    species_data = df.loc[df["Alpha Code"] == x].iloc[0]
                elif input.genus_species():
                    req(any((df["Genus"] + " " + df["Species"] == x)))
                    species_data = df.loc[df["Genus"] + " " + df["Species"] == x].iloc[
                        0
                    ]
                else:
                    req(any((df["Common Name"] == x)))
                    species_data = df.loc[df["Common Name"] == x].iloc[0]
                genus = species_data["Genus"]
                species = species_data["Species"]
                unknown_values = ["unknown", "Unknown", "UNKOWN", "None", "other"]
                if not input.low_data_mode():
                    id_notes = get_summary_for_specimen(genus, species)
                    if genus not in unknown_values and species not in unknown_values:
                        image_url = get_species_image(genus, species, image_number=0)
                        image_url_2 = get_species_image(genus, species, image_number=1)
                    elif genus in unknown_values and species in unknown_values:
                        results = fetch_specimen_classification(
                            genus=genus,
                            species=species,
                            common_name=species_data["Common Name"],
                        )
                        order = results["order"]
                        image_url = get_species_image(
                            order=order, genus=genus, species=species, image_number=0
                        )
                        image_url_2 = get_species_image(
                            order=order, genus=genus, species=species, image_number=1
                        )
                    elif genus not in unknown_values and species in unknown_values:
                        species = ""
                        image_url = get_species_image(genus, species, image_number=0)
                        image_url_2 = get_species_image(genus, species, image_number=1)
                    elif x.lower().startswith("unknown"):
                        results = fetch_specimen_classification(
                            genus=genus,
                            species=species,
                            common_name=species_data["Common Name"],
                        )
                        order = results["order"]
                        image_url = get_species_image(order=order, image_number=0)
                        image_url_2 = get_species_image(order=order, image_number=1)
                    if genus not in unknown_values and species not in unknown_values:
                        m = ui.TagList(
                            ui.markdown(
                                f"### <ins>{species_data['Common Name']}</ins> notes"
                            ),
                            ui.h6("Source: Wikipedia"),
                            id_notes,
                            ui.br(),
                            ui.br(),
                            ui.markdown(
                                f"### <ins>{species_data['Common Name']}</ins> pics"
                            ),
                            ui.h6("Source: iNaturalist"),
                            ui.tags.img(
                                src=image_url, style="width: 100%;height: auto"
                            ),
                            ui.tags.img(
                                src=image_url_2, style="width: 100%;height: auto"
                            ),
                        )
                        notes_val.set(m)

        @render.ui
        def notes():
            return notes_val.get()

        @reactive.Calc
        def data_source():
            df = pd.read_csv(
                f"{csv_dir}/{input.data_source()}.csv", keep_default_na=False
            )
            if "Alpha Code" not in df.columns:
                df["Alpha Code"] = None
            return df

        @reactive.Effect
        def _gps_loc():
            ui.update_text(
                "gps_loc",
                label="GPS Location",
                value=f"{input.latitude()}, {input.longitude()}",
            )

        @output
        @render.text
        def note_accuracy():
            return accuracy_val.get()

        @reactive.Effect
        def _specimen():
            df = data_source()
            if input.alpha_code():
                if "Alpha Code" in df.columns:
                    ui.update_selectize(
                        "specimen",
                        label="Select the specimen observed",
                        choices=list(df["Alpha Code"]),
                        selected="",
                    )
            elif input.genus_species():
                ui.update_selectize(
                    "specimen",
                    label="Select the specimen observed",
                    choices=list(df["Genus"] + " " + df["Species"]),
                    selected="",
                )
            else:
                if "Common Name" in df.columns:
                    ui.update_selectize(
                        "specimen",
                        label="Select the specimen observed",
                        choices=list(df["Common Name"]),
                        selected="",
                    )

        def show_deletion_modal():
            df = val.get()
            index = int(index_to_delete_val.get())
            m = ui.modal(
                ui.br(),
                ui.markdown("### Confirm deletion"),
                ui.markdown(
                    f"Are you sure you want to delete this observation for {df.loc[index, 'Common Name']}?"
                ),
                ui.br(),
                ui.br(),
                ui.layout_columns(
                    ui.input_action_button(
                        "cancel",
                        "Cancel",
                        class_="btn btn-danger",
                        icon=icon_svg("xmark"),
                    ),
                    ui.input_action_button(
                        "confirm_deletion",
                        "Delete",
                        class_="btn btn-success",
                        icon=icon_svg("trash"),
                    ),
                    col_widths=[6, 6],
                ),
                footer=None,
            )
            return m

        @reactive.Effect
        @reactive.event(input.cancel)
        def cancel():
            ui.modal_remove()

        @reactive.Effect
        @reactive.event(input.confirm_deletion)
        def confirm_deletion():
            ui.modal_remove()
            df = val.get()
            index = index_to_delete_val.get()
            df = delete_row_in_df(df, index)
            val.set(df)

            m = ui.modal(
                "Your observation has been cleared",
                easy_close=True,
                footer=None,
            )
            ui.modal_show(m)

        @reactive.Effect
        @reactive.event(input.reset)
        def _reset():
            df = val.get()
            if not df.empty:
                index_to_delete = list(input.observations_data_frame_selected_rows())
                if index_to_delete and index_to_delete[0] in df.index:
                    index_to_delete_val.set(index_to_delete[0])
                    ui.modal_show(show_deletion_modal())

        @reactive.Effect
        @reactive.event(input.upload)
        async def _send_msg_to_reset():
            await session.send_custom_message("clear", "clear")

        @reactive.Effect
        @reactive.event(input.upload)
        async def _upload():
            workbook = get_workbook(survey_config, keyfile_path)
            worksheet = workbook.worksheet(input.google_sheet_selector())
            data = worksheet.get_all_values()

            df = pd.DataFrame(data)

            nan_rows = df.index[df.isnull().all(axis=1)]
            if nan_rows.empty:
                next_row_number = worksheet.row_count
                include_column_header = False
            else:
                next_row_number = int(nan_rows[0])
                include_column_header = True
            df = val.get()
            if not df.empty:
                df = df.drop(
                    columns=[
                        "Data Source",
                        "Bandwidth Saver",
                    ]
                )
                gd.set_with_dataframe(
                    worksheet,
                    df,
                    row=next_row_number + 1,
                    resize=True,
                    include_column_header=include_column_header,
                )
                df = val.get()
                # Clear the DataFrame by creating a new empty one
                df = pd.DataFrame(columns=df.columns)
                val.set(df)
                ui.update_selectize(
                    "survey_side",
                    label="Select the side",
                    choices=survey_config["survey_side"],
                )
                ui.update_text("gps_loc", label="GPS Location", value="")
                ui.update_selectize(
                    "canopy_cover",
                    label="Select the canopy cover",
                    choices=survey_config["canopy_cover"],
                )
                ui.update_accordion_panel(
                    id="accordion_data_sources",
                    target="survey_points",
                    value="survey_points",
                    show=True,
                    icon=icon_svg("location-dot"),
                )
                m = ui.modal(
                    "Your observations have been synced",
                    easy_close=True,
                    footer=None,
                )
                ui.modal_show(m)

        @reactive.Effect
        @reactive.event(input.gps)
        def _gps():
            accuracy_val.set(None)
            ui.insert_ui(
                ui.TagList(
                    ui.tags.script(
                        """
                        $(document).ready(function () {
                            Shiny.setInputValue("accuracy", null);
                            navigator.geolocation.getCurrentPosition(onSuccess, onError, {
                                    enableHighAccuracy: true,
                                    timeout: 5000,
                                    maximumAge: 0
                            });

                            function onError(err) {
                                var errorMessage;
                                switch (err.code) {
                                case err.PERMISSION_DENIED:
                                    errorMessage = "Please approve location access";
                                    break;
                                case err.POSITION_UNAVAILABLE:
                                    errorMessage = "Location information is unavailable";
                                    break;
                                case err.TIMEOUT:
                                    errorMessage = "The request to get user location timed out";
                                    break;
                                default:
                                    errorMessage = "An unknown error occurred";
                                    break;
                                }
                                Shiny.setInputValue("lat", errorMessage);
                            }

                            function onSuccess(position) {
                                var coords = position.coords;
                                Shiny.setInputValue("latitude", coords.latitude);
                                Shiny.setInputValue("longitude", coords.longitude);
                                Shiny.setInputValue("gps_loc", coords.latitude + ", " + coords.longitude);
                                Shiny.setInputValue("accuracy", coords.accuracy);
                            }
                            });
                        """
                    ),
                ),
                selector="#gps",
                where="afterBegin",
            )

        @reactive.Effect
        def _accuracy_val():
            req(input.accuracy())
            accuracy_val.set(f"Accuracy of location: {input.accuracy()} meters")
            ui.update_task_button(id="gps", state="ready")

        @reactive.Effect
        def _genus_species_toggle():
            req(input.genus_species())
            ui.update_switch(id="alpha_code", value=False)

        @reactive.Effect
        def _alpha_code_toggle():
            req(input.alpha_code())
            ui.update_switch(id="genus_species", value=False)

        @reactive.Effect
        @reactive.event(input.submit)
        def _submit():
            if not str(input.surveyors()) == "" and not str(input.specimen()):
                m = (
                    ui.modal(
                        "Surveyor(s) and specimen observed are required fields.",
                        title="Missing required fields",
                        easy_close=True,
                    ),
                )
                ui.modal_show(m)
            req(input.surveyors())
            req(input.specimen())
            df = data_source()
            if input.alpha_code():
                species = df.loc[
                    df["Alpha Code"] == str(input.specimen()), "Species"
                ].values[0]
                genus = df.loc[
                    df["Alpha Code"] == str(input.specimen()), "Genus"
                ].values[0]
                common_name = df.loc[
                    df["Alpha Code"] == str(input.specimen()), "Common Name"
                ].values[0]
            elif input.genus_species():
                species = str(input.specimen()).split(" ")[1]
                genus = str(input.specimen()).split(" ")[0]
                common_name = df.loc[
                    df["Genus"] + " " + df["Species"] == str(input.specimen()),
                    "Common Name",
                ].values[0]
            else:
                species = df.loc[
                    df["Common Name"] == str(input.specimen()), "Species"
                ].values[0]
                genus = df.loc[
                    df["Common Name"] == str(input.specimen()), "Genus"
                ].values[0]
                common_name = str(input.specimen())
            results = fetch_specimen_classification(
                genus=genus, species=species, common_name=common_name
            )
            if input.image_file() and input.image_file() is not None:
                url_list = []
                for image in input.image_file():
                    path = image["datapath"]
                    url = upload_image_to_drive(survey_config, path, keyfile_path)
                    url_list.append(url)
                url_val.set(url_list)
                file_id = url_val.get()[0].split('=')[-1]
                thumbnail_link = f"https://drive.google.com/thumbnail?id={file_id}&sz=640"
                input.image_file().clear()
                ui.remove_ui(selector="div:has(> #image_file-label)")
                ui.insert_ui(
                    ui.input_file(
                        "image_file",
                        "Upload Image File (optional)",
                        accept=[".jpg", ".png", ".jpeg"],
                        multiple=False,
                    ),
                    selector="#submit",
                    where="beforeBegin",
                )
                m = ui.modal(
                    ui.br(),
                    ui.markdown("### Verify your observation"),
                    ui.markdown(f"**Genus:** *{results['genus']}*"),
                    ui.markdown(f"**Species:** *{results['species']}*"),
                    ui.markdown(f"**Common Name:** *{common_name}*"),
                    ui.markdown(f"**Count:** *{str(input.count())}*"),
                    ui.br(),
                    ui.tags.img(src=thumbnail_link, height="100%", width="100%"),
                    ui.br(),
                    ui.br(),
                    ui.layout_columns(
                        ui.input_action_button(
                            "edit",
                            "Edit observation",
                            class_="btn btn-danger",
                            icon=icon_svg("pen-to-square"),
                        ),
                        ui.input_task_button(
                            "submit_verify",
                            "Looks good",
                            class_="btn btn-success",
                            icon=icon_svg("check"),
                            label_busy="Submitting...",
                            icon_busy=loading_tag,
                        ),
                        col_widths=[6, 6],
                    ),
                    footer=None,
                )
            else:
                m = ui.modal(
                    ui.br(),
                    ui.markdown("### Verify your observation"),
                    ui.markdown(f"**Genus:** *{results['genus']}*"),
                    ui.markdown(f"**Species:** *{results['species']}*"),
                    ui.markdown(f"**Common Name:** *{common_name}*"),
                    ui.markdown(f"**Count:** *{str(input.count())}*"),
                    ui.br(),
                    ui.layout_columns(
                        ui.input_action_button(
                            "edit",
                            "Edit observation",
                            class_="btn btn-danger",
                            icon=icon_svg("pen-to-square"),
                        ),
                        ui.input_task_button(
                            "submit_verify",
                            "Looks good",
                            class_="btn btn-success",
                            icon=icon_svg("check"),
                            label_busy="Submitting...",
                            icon_busy=loading_tag,
                        ),
                        col_widths=[6, 6],
                    ),
                    footer=None,
                )
            ui.modal_show(m)

        def delete_row_in_df(df, index):
            df.drop(index, inplace=True)
            df = pd.DataFrame(df)
            # Reindex the DataFrame
            df.reset_index(drop=True, inplace=True)
            return df

        @output
        @render.ui
        def observations_list():
            m = []
            for index, row in val.get().iterrows():
                m.append(get_card_data(index=index, row=row))
            return ui.TagList(
                m,
            )

        def get_card_data(index, row):
            date_observed = row["Date observed"]
            location = row["Location"]
            plot = row["Plot"]
            surveyors = row["Surveyors"]
            survey_point = row["Survey Point"]
            side = row["Side"]
            common_name = row["Common Name"]
            count = row["Count"]
            notes = row["Notes"]
            canopy_cover = row["Canopy cover"]
            life_stage = row["Life stage"]
            height = row["Height (cm)"]
            abundance = row["Abundance"]
            if str(input.plant_survey()) == "Yes":
                return ui.card(
                    ui.card_header(
                        ui.h4(f"{common_name}"),
                    ),
                    ui.markdown(f"**Date observed:** {date_observed}"),
                    ui.markdown(f"**Location:** {location}"),
                    ui.markdown(f"**Plot:** {plot}"),
                    ui.markdown(f"**Survey Point:** {survey_point}"),
                    ui.markdown(f"**Side:** {side}"),
                    ui.markdown(f"**Count:** {count}"),
                    ui.markdown(f"**Notes:** {notes}"),
                    ui.markdown(f"**Canopy cover:** {canopy_cover}"),
                    ui.markdown(f"**Life stage:** {life_stage}"),
                    ui.markdown(f"**Height (cm):** {height}"),
                    ui.markdown(f"**Abundance:** {abundance}"),
                    ui.input_action_button(
                        f"delete_{index}",
                        label="",
                        icon=icon_svg("trash"),
                        class_="btn btn-outline-danger",
                        width="px",
                    ),
                )
            else:
                return ui.card(
                    ui.card_header(
                        ui.h4(f"{common_name}"),
                    ),
                    ui.markdown(f"**Date observed:** {date_observed}"),
                    ui.markdown(f"**Location:** {location}"),
                    ui.markdown(f"**Plot:** {plot}"),
                    ui.markdown(f"**Survey Point:** {survey_point}"),
                    ui.markdown(f"**Side:** {side}"),
                    ui.markdown(f"**Count:** {count}"),
                    ui.markdown(f"**Notes:** {notes}"),
                    ui.input_action_button(
                        f"delete_{index}",
                        label="",
                        icon=icon_svg("trash"),
                        class_="btn btn-outline-danger",
                    ),
                )

        delete_hooks: list[reactive.Effect] = []

        @reactive.effect
        @reactive.event(val)
        def _delete_hooks():
            def make_hook(index):
                @reactive.Effect
                @reactive.event(input[f"delete_{index}"])
                def delete_row():
                    index_to_delete_val.set(index)
                    ui.modal_show(show_deletion_modal())

                return delete_row

            for index in range(len(val.get().index)):
                if index >= len(delete_hooks):
                    delete_hooks.append(make_hook(index))

        @output
        @render.data_frame
        def observations_data_frame():
            if str(input.plant_survey()) == "Yes":
                return render.DataGrid(
                    val.get()[
                        [
                            "Date observed",
                            "Location",
                            "Plot",
                            "Surveyors",
                            "Survey Point",
                            "Side",
                            "Common Name",
                            "Count",
                            "Notes",
                            "Canopy cover",
                            "Life stage",
                            "Height (cm)",
                            "Abundance",
                        ]
                    ],
                    row_selection_mode="single",
                )
            else:
                return render.DataGrid(
                    val.get()[
                        [
                            "Date observed",
                            "Location",
                            "Plot",
                            "Survey Point",
                            "Side",
                            "Common Name",
                            "Count",
                            "Notes",
                        ]
                    ],
                    row_selection_mode="single",
                )

        @reactive.Effect
        @reactive.event(input.restore)
        async def _send_msg_to_restore():
            await session.send_custom_message("restore", "restore")

        @reactive.Effect
        @reactive.event(input.restore_df)
        async def _restore_observations():
            df_string = input.restore_df()
            req(df_string)
            newValue = val.get()
            if df_string and newValue.empty:
                df_json = json.loads(df_string)
                restored_val = str(df_json)
                restored_df = pd.DataFrame(ast.literal_eval(restored_val))
                if not newValue.equals(restored_df):
                    ui.update_selectize(
                        id="surveyors",
                        label="Choose Surveyor(s)*:",
                        choices=survey_config["surveyors"],
                        selected=restored_df.iloc[-1]["Surveyors"].split(", "),
                    )
                    ui.update_selectize(
                        id="location",
                        label="Select the location",
                        choices=survey_config["location"],
                        selected=restored_df.iloc[-1]["Location"],
                    )
                    ui.update_selectize(
                        id="plot",
                        label="Select the plot",
                        choices=survey_config["plots"],
                        selected=restored_df.iloc[-1]["Plot"],
                    )
                    ui.update_selectize(
                        id="survey_point",
                        label="Select the survey point",
                        choices=survey_config["survey_points"],
                        selected=restored_df.iloc[-1]["Survey Point"],
                    )
                    ui.update_selectize(
                        id="survey_side",
                        label="Select the side",
                        choices=survey_config["survey_side"],
                        selected=restored_df.iloc[-1]["Side"],
                    )
                    ui.update_switch(
                        id="low_data_mode",
                        label="Bandwidth Saver",
                        value=restored_df.iloc[-1]["Bandwidth Saver"] == "True",
                    )
                    ui.update_radio_buttons(
                        id="data_source",
                        label="Select data source",
                        choices=survey_config["survey_data_sources"],
                        selected=restored_df.iloc[-1]["Data Source"],
                    )

                    ui.update_accordion_panel(
                        id="accordion_data_sources",
                        target="data_source",
                        title=f"Data source: {restored_df.iloc[-1]['Data Source']}",
                        value="data_source",
                        show=False,
                    )

                    ui.update_accordion_panel(
                        id="accordion_data_sources",
                        target="specimen_details",
                        value="specimen_details",
                        show=True,
                    )

                    ui.update_accordion_panel(
                        id="accordion_data_sources",
                        target="surveyors",
                        title="Surveyors: " + (restored_df.iloc[-1]["Surveyors"]),
                        value="surveyors",
                        show=False,
                    )

                    ui.update_accordion_panel(
                        id="accordion_data_sources",
                        target="survey_points",
                        title="Survey Point: "
                        + restored_df.iloc[-1]["Plot"]
                        + f" - {restored_df.iloc[-1]['Survey Point']}",
                        value="survey_points",
                        show=False,
                    )

                    val.set(restored_df)
                m = ui.modal(
                    "Your observations have been restored",
                    easy_close=True,
                    footer=None,
                )
                ui.modal_show(m)
            else:
                m = ui.modal(
                    "Nothing to restore",
                    easy_close=True,
                    footer=None,
                )
                ui.modal_show(m)
            await session.send_custom_message("restore", "complete")

        @reactive.Effect
        @reactive.event(input.next_screen)
        def _next_page():
            ui.update_navs("page_navbar", selected="verify")

        @reactive.Effect
        @reactive.event(input.previous_screen)
        def _prev_page():
            ui.update_navs("page_navbar", selected="record")

        @reactive.Effect
        @reactive.event(input.edit)
        def _edit_observation():
            ui.modal_remove()

        @reactive.Effect
        @reactive.event(input.submit_verify)
        def _submit_observation():
            df = data_source()
            if input.gps_loc():
                latitude = input.latitude()
                longitude = input.longitude()
                gps_location = input.gps_loc()
            else:
                # use the default location of the SF bay for weather if user did not provide
                latitude = "37.4803"
                longitude = "-122.0786"
                gps_location = "None given"
            if survey_config.get("timezone"):
                tz = pytz.timezone(survey_config["timezone"])
                now = datetime.datetime.now(tz)
            else:
                tf = TimezoneFinder()
                now = datetime.datetime.now(
                    pytz.timezone(
                        tf.certain_timezone_at(
                            lng=float(longitude), lat=float(latitude)
                        )
                    )
                )
            if str(input.plant_survey()) == "Yes":
                canopy_cover = str(input.canopy_cover())
                life_stage = str(input.life_stage())
                height = str(input.height())
                abundance = str(input.abundance())
            else:
                canopy_cover = None
                life_stage = None
                height = None
                abundance = None

            if input.alpha_code():
                alpha_code = str(input.specimen())
                common_name = df.loc[
                    df["Alpha Code"] == str(input.specimen()), "Common Name"
                ].values[0]
                species = df.loc[
                    df["Alpha Code"] == str(input.specimen()), "Species"
                ].values[0]
                genus = df.loc[
                    df["Alpha Code"] == str(input.specimen()), "Genus"
                ].values[0]
            elif input.genus_species():
                species = str(input.specimen()).split(" ")[1]
                genus = str(input.specimen()).split(" ")[0]
                alpha_code = df.loc[
                    df["Genus"] + " " + df["Species"] == str(input.specimen()),
                    "Alpha Code",
                ].values[0]
                common_name = df.loc[
                    df["Genus"] + " " + df["Species"] == str(input.specimen()),
                    "Common Name",
                ].values[0]
            else:
                species = df.loc[
                    df["Common Name"] == str(input.specimen()), "Species"
                ].values[0]
                genus = df.loc[
                    df["Common Name"] == str(input.specimen()), "Genus"
                ].values[0]
                alpha_code = df.loc[
                    df["Common Name"] == str(input.specimen()), "Alpha Code"
                ].values[0]
                common_name = str(input.specimen())
            results = fetch_specimen_classification(
                genus=genus, species=species, common_name=common_name
            )
            data = {
                "fields": {
                    "Bandwidth Saver": str(input.low_data_mode()),
                    "Data Source": str(input.data_source()),
                    "Date observed": str(input.survey_date()),
                    "Time observed": str(now.strftime("%I:%M %p")),
                    "Location": str(input.location()),
                    "Plot": str(input.plot()),
                    "Survey Point": str(input.survey_point()),
                    "Side": str(input.survey_side()),
                    "Genus": results["genus"],
                    "Species": results["species"],
                    "Class": results["class"],
                    "Order": results["order"],
                    "Family": results["family"],
                    "Common Name": common_name,
                    "Alpha Code": alpha_code,
                    "Count": str(input.count()),
                    "Notes": str(input.notes()),
                    "Surveyors": str(", ".join(input.surveyors())),
                    "Url": (
                        ", ".join(url_val.get()) if url_val.get() is not None else ""
                    ),
                    "Weather": get_weather_underground_temperature(
                        str(latitude), str(longitude)
                    ),
                    "GPS Location": str(gps_location),
                    "Canopy cover": canopy_cover,
                    "Life stage": life_stage,
                    "Height (cm)": height,
                    "Abundance": abundance,
                }
            }

            newValue = val.get()
            newValue = newValue._append(data["fields"], ignore_index=True)
            val.set(newValue)
            ui.notification_show(
                "Your observation has been recorded.", duration=2, type="message"
            )
            if input.alpha_code():
                ui.update_selectize(
                    "specimen",
                    label="Select the specimen observed",
                    choices=list(df["Alpha Code"]),
                    selected="",
                )
            elif input.genus_species():
                ui.update_selectize(
                    "specimen",
                    label="Select the specimen observed",
                    choices=list(df["Genus"] + " " + df["Species"]),
                    selected="",
                )
            else:
                ui.update_selectize(
                    "specimen",
                    label="Select the specimen observed",
                    choices=list(df["Common Name"]),
                    selected="",
                )
            ui.update_selectize(
                "count",
                label="Count observed",
                choices=survey_config["count"],
            )
            ui.update_selectize(
                "life_stage",
                label="Select the life stage",
                choices=survey_config["life_stage"],
            )
            ui.update_numeric(
                "height",
                label="Enter the height of the plant (in cm)",
                value=0,
                min=0,
                max=1000,
                step=5,
            )
            ui.update_selectize(
                "abundance",
                label="Select the abundance",
                choices=survey_config["abundance_category"],
            )
            ui.update_text("notes", label="Notes", value="")
            accuracy_val.set(None)
            ui.modal_remove()
            url_val.set(None)

        @reactive.Effect
        @reactive.event(input.submit_verify)
        async def _submit_msg():
            await session.send_custom_message(
                "df", str(val.get().to_json(orient="records"))
            )

        @reactive.Effect
        @reactive.event(input.go_to_surveyors)
        def _go_to_surveyors():
            ui.update_accordion_panel(
                id="accordion_data_sources",
                target="surveyors",
                value="surveyors",
                show=True,
            )
            ui.update_accordion_panel(
                id="accordion_data_sources",
                target="data_source",
                title=f"Data source: {input.data_source()}",
                value="data_source",
                show=False,
            )

        @reactive.Effect
        @reactive.event(input.go_to_survey_points)
        def _go_to_survey_points():
            req(input.surveyors())
            ui.update_accordion_panel(
                id="accordion_data_sources",
                target="survey_points",
                value="survey_points",
                show=True,
            )
            ui.update_accordion_panel(
                id="accordion_data_sources",
                target="surveyors",
                title="Surveyors: " + str(", ".join(input.surveyors())),
                value="surveyors",
                show=False,
            )

        @reactive.Effect
        @reactive.event(input.go_to_specimen)
        def _go_to_specimen():
            req(input.survey_point())
            ui.update_accordion_panel(
                id="accordion_data_sources",
                target="specimen_details",
                value="specimen_details",
                show=True,
            )
            ui.update_accordion_panel(
                id="accordion_data_sources",
                target="survey_points",
                title="Survey Point: "
                + f"{str(input.plot())} - {str(input.survey_point())}",
                value="survey_points",
                show=False,
            )

    app_dir = Path(__file__).parent
    app = App(app_ui, server, static_assets=app_dir / "www")
    return app



from setuptools import setup, find_packages

setup(
    name="fieldsurveys",
    version="0.1.37",
    author="Karan Gathani",
    author_email="karan.gathani+fieldsurveysapp@posit.co",
    packages=find_packages(),
    description="A Python package for creating and managing field surveys.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/karangattu/fieldsurveys",
    include_package_data=True,
    package_data={"fieldsurveys": ["*"]},
    entry_points={"console_scripts": ["fieldsurveys = fieldsurveys:copy_app_files"]},
    setup_requires=["wheel"],
    install_requires=[
        "beautifulsoup4==4.12.3",
        "colorama==0.4.6",
        "cryptography==42.0.5",
        "faicons==0.2.2",
        "geopy==2.4.1",
        "google-api-python-client==2.120.0",
        "google-auth-httplib2==0.2.0",
        "google-auth-oauthlib==1.2.0",
        "google-auth==2.28.1",
        "gspread-dataframe==3.3.1",
        "gspread==6.0.2",
        "htmltools==0.5.1",
        "pandas==2.2.1",
        "plotly==5.19.0",
        "pytz==2024.1",
        "PyYAML==6.0.1",
        "questionary==2.0.1",
        "requests==2.31.0",
        "rsconnect_python==1.22.0",
        "shiny==0.7.1",
        "shinyswatch==0.4.2",
        "timezonefinder==6.4.1",
        "us==3.1.1",
        "Wikipedia-API==0.6.0",
    ],
    extras_require={
        "tests": [
            "playwright==1.41.2",
            "pytest==8.0.2",
            "pytest-mock==3.12.0",
            "pytest-playwright==0.4.4",
        ]
    },
)



import requests
from google.oauth2 import service_account
from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload
import time


def upload_image_to_imgur(survey_config, image_path):
    client_id = survey_config["imgur_app_id"]
    # Imgur API endpoint
    api_url = "https://api.imgur.com/3/upload"

    headers = {
        "Authorization": f"Client-ID {client_id}",
    }

    with open(image_path, "rb") as file:
        files = {"image": file}
        data = {"type": "file"}

        response = requests.post(api_url, headers=headers, files=files, data=data)

        # Check if the request was successful
        if response.status_code == 200:
            return response.json()["data"]["link"]


def upload_image_to_drive(survey_config, image_path, keyfile_json):
    SCOPES = ["https://www.googleapis.com/auth/drive"]

    creds = service_account.Credentials.from_service_account_file(
        keyfile_json, scopes=SCOPES
    )

    service = build("drive", "v3", credentials=creds)

    folder_id = survey_config["google_folder_id"]
    # Upload the image
    file_metadata = {"parents": [folder_id]}
    media = MediaFileUpload(image_path, mimetype="image/jpeg", resumable=True)
    # Generate a unique name for the file based on current time
    file_name = f"image_{int(time.time())}.jpg"

    file_metadata = {"name": file_name, "parents": [folder_id]}
    media = MediaFileUpload(image_path, mimetype="image/jpeg", resumable=True)

    file = (
        service.files()
        .create(body=file_metadata, media_body=media, fields="id")
        .execute()
    )

    # Set the permissions to make the image publicly accessible
    service.permissions().create(
        fileId=file["id"], body={"role": "reader", "type": "anyone"}
    ).execute()

    # Get the URL of the uploaded image
    file_url = f"https://drive.google.com/uc?id={file['id']}"

    return file_url
